#include <iostream>

using namespace std;

#include <CL/opencl.h>

const char* program_source[] =
{
	"__kernel void IntAdd(__global const int* a, __global const int* b, __global int* c)",
	"{",
		"int iGID = get_global_id(0);",
		"c[iGID] = a[iGID] + b[iGID];",
	"}",
};


int main(int argc, char **argv)
{
	const int dimension = 3;
	const cl_uint num_entries = 10;
	cl_platform_id platforms[num_entries];
	cl_uint num_platforms;
	clGetPlatformIDs(num_entries, platforms, &num_platforms);
	
	// Obtenir les devices de la premire 
	// plate-forme trouve
	cl_device_id devices[num_entries];
	cl_uint num_devices;
	clGetDeviceIDs(platforms[0], CL_DEVICE_TYPE_GPU, num_entries, devices, &num_devices);


	// La table de proprits pour la fonction
	// clCreateContext
	cl_context_properties properties[] =
	{ 
		CL_CONTEXT_PLATFORM,
		(cl_context_properties)platforms[0], // la premire plate-forme
		0 
	};

	
	cl_int error;
	cl_context context = clCreateContext(properties,
							num_devices, devices,
							NULL, NULL, &error);

	if(error != CL_SUCCESS)
	{
		cerr << "Erreur clCreateContext" << endl;
		return 1;
	}

	cl_command_queue command_queue = clCreateCommandQueue (context,
										devices[0],
										NULL, &error);
	if(error != CL_SUCCESS)
	{
		cerr << "Erreur clCreateCommandQueue" << endl;
		clReleaseContext(context);
		return 1;
	}

	cl_int a[] = {1, 2, 3};
	cl_int b[] = {4, 5, 6};
	cl_int c[] = {0, 0, 0}; // le rsultat de l'addition sera stock ici

	// cration des buffers de donnes
	// REMARQUE:  partir d'ici, on ne gre plus les erreurs
	// afin d'allger le code et de gagner en clart
	
	// Buffer pour les donnes a
	cl_mem buffera = clCreateBuffer (context,
		CL_MEM_READ_ONLY,
		sizeof(cl_int)*dimension,
		NULL,
		&error);

	// Buffer pour les donnes b
	cl_mem bufferb = clCreateBuffer (context,
		CL_MEM_READ_ONLY,
		sizeof(cl_int)*dimension,
		NULL,
		&error);

	// Buffer pour les resultats c
	cl_mem bufferc = clCreateBuffer (context,
		CL_MEM_WRITE_ONLY,
		sizeof(cl_int)*dimension,
		NULL,
		&error);

	cl_program program = clCreateProgramWithSource(
		context,
		sizeof(program_source)/sizeof(char*),
		program_source,
		NULL,
		&error);
	
	cl_int result = clBuildProgram(program, 0, NULL, NULL, NULL, NULL);
	cl_kernel kernel = clCreateKernel(program, "IntAdd", NULL);

	result = clSetKernelArg(kernel, 0, sizeof(cl_mem), &buffera);
	result = clSetKernelArg(kernel, 1, sizeof(cl_mem), &bufferb);
	result = clSetKernelArg(kernel, 2, sizeof(cl_mem), &bufferc);

	const size_t global_work_size[] = {
		dimension,
	};

	// Mettre le buffer a dans la queue et
	// copier les donnes de faon asynchrone
	result = clEnqueueWriteBuffer (command_queue,
		buffera,
		CL_FALSE,
		0,
		sizeof(cl_int)*dimension,
		&a,
		0, NULL, NULL);

	// Mettre le buffer a dans la queue et
	// copier les donnes de faon asynchrone
	result = clEnqueueWriteBuffer (command_queue,
		bufferb,
		CL_FALSE,
		0,
		sizeof(cl_int)*dimension,
		&b,
		0, NULL, NULL);

	result = clEnqueueNDRangeKernel (command_queue,
					kernel,
					1,
					NULL,
					global_work_size,
					NULL, 0, NULL, NULL);

	result = clEnqueueReadBuffer(command_queue,
		bufferc,
		CL_TRUE,
		0,
		sizeof(cl_int)*dimension,
		&c,
		0, NULL, NULL);

	cout << "Resultat: " << endl;
	for(int i=0; i<3; i++)
		cout << c[i] << endl;

	clReleaseKernel(kernel);
	clReleaseProgram(program);
	clReleaseMemObject(bufferc);
	clReleaseMemObject(bufferb);
	clReleaseMemObject(buffera);
	clReleaseCommandQueue(command_queue);		
	clReleaseContext(context);
}
 

