// DemoProfiling.cpp : Defines the entry point for the console application.
//


#include <iostream>

using namespace std;

#include <CL/opencl.h>

const char* program_source[] =
{
	"__kernel void Hello()",
	"{",
		"long i;",
		"float f;",
		"for(i=0 ; i<10000000; i++)",
		"	f = sin(0.5) + cos (0.4) + tan (0.3);",	
	"}",
};


int main(int argc, const char** argv) 
{
	cl_int result;
	
	// Obtenir les identifiants de plates-formes
	const cl_uint num_entries = 10;
	cl_platform_id platforms[num_entries];
	cl_uint num_platforms;
	result = clGetPlatformIDs(num_entries, platforms, &num_platforms);
	if(result != CL_SUCCESS)
	{
			cerr << "Erreur clGetPlatformIDs" << endl;
			return 1;
	}
	cout << num_platforms << " plates-formes detectees" << endl << endl;

	// Obtenir les infos sur les plates-formes
	const size_t size_buffer = 255;
	char buffer[size_buffer+1];
	size_t param_value_size_ret;
	for(cl_uint i=0; i< num_platforms; i++)
	{
		clGetPlatformInfo(platforms[i], CL_PLATFORM_PROFILE, size_buffer, buffer, &param_value_size_ret);
		cout << "CL_PLATFORM_PROFILE: " << buffer << endl;

		clGetPlatformInfo(platforms[i], CL_PLATFORM_VERSION, size_buffer, buffer, &param_value_size_ret);
		cout << "CL_PLATFORM_VERSION: " << buffer << endl;

		clGetPlatformInfo(platforms[i], CL_PLATFORM_NAME, size_buffer, buffer, &param_value_size_ret);
		cout << "CL_PLATFORM_NAME: " << buffer << endl;
		
		clGetPlatformInfo(platforms[i], CL_PLATFORM_VENDOR, size_buffer, buffer, &param_value_size_ret);
		cout << "CL_PLATFORM_VENDOR: " << buffer << endl;
	/*	
		cout << endl;
		clGetPlatformInfo(platforms[i], CL_PLATFORM_EXTENSIONS, size_buffer, buffer, &param_value_size_ret);
		cout << "CL_PLATFORM_EXTENSIONS: " << buffer << endl;
*/
	}

	// Obtenir les informations sur les devices de
	// la plate-forme  l'indice 0

	cl_device_id devices[num_entries];
	cl_uint num_devices;

	cout  << endl << "-------------------------" << endl << endl;
	result = clGetDeviceIDs(platforms[0], CL_DEVICE_TYPE_ALL, num_entries, devices, &num_devices);
	if(result != CL_SUCCESS)
	{
			cerr << "Erreur clGetDeviceIDs" << endl;
			return 1;
	}
	for(cl_uint i=0; i< num_devices; i++)
	{
		clGetDeviceInfo(devices[i], CL_DEVICE_TYPE, size_buffer, buffer, &param_value_size_ret);
		int value = (int)buffer[0];
		if(value & CL_DEVICE_TYPE_CPU)
			cout << "CL_DEVICE_TYPE: " << "CL_DEVICE_TYPE_CPU" << endl;
		if(value & CL_DEVICE_TYPE_GPU)
			cout << "CL_DEVICE_TYPE: " << "CL_DEVICE_TYPE_GPU" << endl;

		clGetDeviceInfo(devices[i], CL_DEVICE_NAME, size_buffer, buffer, &param_value_size_ret);
		cout << "CL_DEVICE_NAME: " << buffer << endl;

		clGetDeviceInfo(devices[i], CL_DEVICE_VENDOR, size_buffer, buffer, &param_value_size_ret);
		cout << "CL_DEVICE_VENDOR: " << buffer << endl;

		clGetDeviceInfo(devices[i], CL_DEVICE_COMPILER_AVAILABLE, size_buffer, buffer, &param_value_size_ret);
		value = (int)buffer[0];
		if(value == 1)
			cout << "CL_DEVICE_COMPILER_AVAILABLE: " << " ok" << endl;

		size_t resolution;
		clGetDeviceInfo(devices[i], CL_DEVICE_PROFILING_TIMER_RESOLUTION, sizeof(size_t), &resolution, &param_value_size_ret);
		if(value == 1)
			cout << "CL_DEVICE_PROFILING_TIMER_RESOLUTION: " << resolution << endl;

		cout << endl << "-------------------------" << endl;
	}


	cout << "Demo profiling" << endl;
	// La table de proprits pour la fonction
	// clCreateContext
	cl_context_properties properties[] =
	{ 
		CL_CONTEXT_PLATFORM,
		(cl_context_properties)platforms[0], // la premire plate-forme
		0 
	};

	
	cl_int error;
	cl_context context = clCreateContext(properties,
							num_devices, devices,
							NULL, NULL, &error);

	if(error != CL_SUCCESS)
	{
		cerr << "Erreur clCreateContext" << endl;
		return 1;
	}

	cl_command_queue command_queue = clCreateCommandQueue (context,
										devices[0],
										CL_QUEUE_PROFILING_ENABLE, &error);

	if(error != CL_SUCCESS)
	{
		cerr << "Erreur clCreateCommandQueue" << endl;
		clReleaseContext(context);
		return 1;
	}

	cl_program program = clCreateProgramWithSource(
		context,
		sizeof(program_source)/sizeof(char*),
		program_source,
		NULL,
		&error);

	
	result = clBuildProgram(program, 0, NULL, NULL, NULL, NULL);
	cl_kernel kernel_hello = clCreateKernel(program, "Hello", NULL);


	
	const size_t global_work_size_hello[] = {
		1,
	};
	cl_event eventa;
	cl_event event_list[1];
	cl_uint num = sizeof(event_list)/sizeof(cl_event);

	result = clEnqueueNDRangeKernel (command_queue,
					kernel_hello,
					1,
					NULL,
					global_work_size_hello,
					NULL, 0, NULL, &eventa);
	event_list[0] = eventa;

	result = clWaitForEvents (num, event_list);

	cl_ulong value, start, end, temps;

	result = clGetEventProfilingInfo (eventa,
		CL_PROFILING_COMMAND_QUEUED,
		sizeof(cl_ulong),
		&value,
		NULL);

	cout << "CL_PROFILING_COMMAND_QUEUED: " << value << endl;

	result = clGetEventProfilingInfo (eventa,
		CL_PROFILING_COMMAND_SUBMIT,
		sizeof(cl_ulong),
		&value,
		NULL);

	cout << "CL_PROFILING_COMMAND_SUBMIT: " << value << endl;

		result = clGetEventProfilingInfo (eventa,
		CL_PROFILING_COMMAND_START,
		sizeof(cl_ulong),
		&value,
		NULL);

	cout << "CL_PROFILING_COMMAND_START: " << value << endl;
	start = value;

	result = clGetEventProfilingInfo (eventa,
		CL_PROFILING_COMMAND_END,
		sizeof(cl_ulong),
		&value,
		NULL);

	cout << "CL_PROFILING_COMMAND_END: " << value << endl;
	end = value;
	temps = end - start;
	double ftemps = (double)temps;
	cout << "Temps d'execution du kernel: " << temps << " nanosecondes" << endl;
	cout << "soit: " << temps / 1000000000.0 << " secondes" << endl;

	cout << "Fin Demo profiling" << endl;

	clReleaseEvent(eventa);
	clReleaseKernel(kernel_hello);
	clReleaseProgram(program);
	clReleaseCommandQueue(command_queue);		
	clReleaseContext(context);

}
