// DemoWaitForEvent.cpp : Defines the entry point for the console application.
//

#include <iostream>

using namespace std;

#include <CL/opencl.h>

const char* program_source[] =
{
	"__kernel void Hello( __global const char* a, __global char* tempa)",
	"{",
		"long i;",
		"float f;",
		"for(i=0 ; i<10000000; i++)",
		"	f = sin(0.5) + cos (0.4) + tan (0.3);",	
		"int iGID = get_global_id(0);",
		"tempa[iGID] = a[iGID];",
	"}",

	"__kernel void World(__global const char* tempa, __global const char* b, __global char* tempb)",
	"{",
		"int iGID = get_global_id(0);",
		"tempb[iGID] = tempa[iGID];",
		"tempb[iGID+6] = b[iGID];",
	"}",
	
	"__kernel void HelloWorld(__global const char* tempb, __global char* c)",
	"{",
		"int iGID = get_global_id(0);",
		"c[iGID] = tempb[iGID];",
	"}",
};



int main(int argc, char **argv)
{
	const cl_uint num_entries = 10;
	cl_platform_id platforms[num_entries];
	cl_uint num_platforms;
	clGetPlatformIDs(num_entries, platforms, &num_platforms);

	cout << "Demo WaitForEvents" << endl;
	
	// Obtenir les devices de la premire 
	// plate-forme trouve
	cl_device_id devices[num_entries];
	cl_uint num_devices;
	clGetDeviceIDs(platforms[0], CL_DEVICE_TYPE_GPU, num_entries, devices, &num_devices);


	// La table de proprits pour la fonction
	// clCreateContext
	cl_context_properties properties[] =
	{ 
		CL_CONTEXT_PLATFORM,
		(cl_context_properties)platforms[0], // la premire plate-forme
		0 
	};

	
	cl_int error;
	cl_context context = clCreateContext(properties,
							num_devices, devices,
							NULL, NULL, &error);

	if(error != CL_SUCCESS)
	{
		cerr << "Erreur clCreateContext" << endl;
		return 1;
	}

	cl_command_queue command_queue = clCreateCommandQueue (context,
										devices[0],
										NULL, &error);

	if(error != CL_SUCCESS)
	{
		cerr << "Erreur clCreateCommandQueue" << endl;
		clReleaseContext(context);
		return 1;
	}

	const cl_char a[] = "Hello ";
	const cl_char b[] = "World!";
	cl_char c[] = "------------"; // le rsultat de l'addition sera stock ici

	// cration des buffers de donnes
	// REMARQUE:  partir d'ici, on ne gre plus les erreurs
	// afin d'allger le code et de gagner en clart
	
	// Buffer pour la donne a
	cl_mem buffera = clCreateBuffer (context,
		CL_MEM_READ_ONLY | CL_MEM_COPY_HOST_PTR,
		sizeof(a)-1, // pas de zro final dans le buffer
		(void*)&a,
		&error);

	// Buffer temporaire a
	cl_mem buffertempa = clCreateBuffer (context,
		CL_MEM_READ_WRITE | CL_MEM_COPY_HOST_PTR,
		sizeof(a)-1, // pas de zro final dans le buffer
		(void*)&c,
		&error);

	// Buffer pour la donne b
	cl_mem bufferb = clCreateBuffer (context,
		CL_MEM_READ_ONLY | CL_MEM_COPY_HOST_PTR,
		sizeof(b)-1, // pas de zro final dans le buffer
		(void*)&b,
		&error);

	// Buffer temporaire b
	cl_mem buffertempb = clCreateBuffer (context,
		CL_MEM_READ_WRITE | CL_MEM_COPY_HOST_PTR,
		sizeof(c)-1, // pas de zro final dans le buffer
		(void*)&c,
		&error);


	// Buffer pour la donnes resultat c
	cl_mem bufferc = clCreateBuffer (context,
		CL_MEM_WRITE_ONLY,
		sizeof(c)-1, // Pas de zro final dans le buffer
		(void*)&c,
		&error);

	cl_program program = clCreateProgramWithSource(
		context,
		sizeof(program_source)/sizeof(char*),
		program_source,
		NULL,
		&error);
	
	cl_int result = clBuildProgram(program, 0, NULL, NULL, NULL, NULL);
	cl_kernel kernel_hello = clCreateKernel(program, "Hello", NULL);
	cl_kernel kernel_world = clCreateKernel(program, "World", NULL);
	cl_kernel kernel_helloworld = clCreateKernel(program, "HelloWorld", NULL);


	result = clSetKernelArg(kernel_hello, 0, sizeof(cl_mem), &buffera);
	result = clSetKernelArg(kernel_hello, 1, sizeof(cl_mem), &buffertempa);

	result = clSetKernelArg(kernel_world, 0, sizeof(cl_mem), &buffertempa);
	result = clSetKernelArg(kernel_world, 1, sizeof(cl_mem), &bufferb);
	result = clSetKernelArg(kernel_world, 2, sizeof(cl_mem), &buffertempb);

	result = clSetKernelArg(kernel_helloworld, 0, sizeof(cl_mem), &buffertempb);
	result = clSetKernelArg(kernel_helloworld, 1, sizeof(cl_mem), &bufferc);

	result =  clSetCommandQueueProperty(command_queue,
		CL_QUEUE_OUT_OF_ORDER_EXEC_MODE_ENABLE,
		CL_TRUE,
		NULL);

	const size_t global_work_size_hello[] = {
		sizeof(a)-1,
	};

	const size_t global_work_size_world[] = {
		sizeof(b)-1,
	};

	const size_t global_work_size_helloworld[] = {
		sizeof(c)-1,
	};

	cl_event eventa, eventb, eventc;
	cl_event event_list[1];

	result = clEnqueueNDRangeKernel (command_queue,
					kernel_hello,
					1,
					NULL,
					global_work_size_hello,
					NULL, 0, NULL, &eventa);

	event_list[0] = eventa;
	cl_uint num = sizeof(event_list)/sizeof(cl_event);

	result = clWaitForEvents (num, event_list);


	result = clEnqueueNDRangeKernel (command_queue,
					kernel_world,
					1,
					NULL,
					global_work_size_world,
					NULL, 0, NULL, &eventb);


	event_list[0] = eventb;
	result = clWaitForEvents (num, event_list);



	result = clEnqueueNDRangeKernel (command_queue,
					kernel_helloworld,
					1,
					NULL,
					global_work_size_helloworld,
					NULL, 0, NULL, &eventc);

	event_list[0] = eventc;
	result = clWaitForEvents (num, event_list);

	result = clEnqueueReadBuffer(command_queue,
		bufferc,
		CL_TRUE,
		0,
		sizeof(c)-1,
		&c,
		0, NULL, NULL);

	cout << "Resultat: " << c << endl;

	result = clReleaseEvent(eventc);
	result = clReleaseEvent(eventb);
	result = clReleaseEvent(eventa);
	
	clReleaseKernel(kernel_hello);
	clReleaseKernel(kernel_world);
	clReleaseKernel(kernel_helloworld);
	clReleaseProgram(program);


	clReleaseMemObject(bufferc);
	clReleaseMemObject(buffertempb);
	clReleaseMemObject(bufferb);
	clReleaseMemObject(buffertempa);
	clReleaseMemObject(buffera);
	clReleaseCommandQueue(command_queue);		
	clReleaseContext(context);

	//cin.get();
}

